import math
from typing import Union, Tuple

import numpy as np

from gui.layer import CharactersLayer
from gui.impl.opengl import OpenGLGUIFacade
from .OpenGLLayer import OpenGLLayer


class OpenGLCharactersLayer(OpenGLLayer, CharactersLayer):

    def __init__(self, gui: OpenGLGUIFacade):
        super(OpenGLCharactersLayer, self).__init__(gui)
        self.__width = 0  # type: int
        self.__height = 0  # type: int
        self.__depthMax = 0  # type: int
        self.__depthStep = 0  # type: int

    def updateDepths(self, depth: int, index: int, maxIndex: int) -> int:
        self.__depthMax = depth - index
        self.__depthStep = maxIndex
        self.setDepths(self.__depthMax)
        return self.__depthMax - self.__height * self.__depthStep

    def setGridSize(self, width: int, height: int):
        self.__width = width
        self.__height = height

    def getCharacterCount(self) -> int:
        return self.faceCount

    def setCharacterCount(self, count: int):
        self.createFaces(count)

    def getCharacterLocation(self, charIndex) -> (float, float):
        return self.getFaceTileLocation(charIndex)

    def setCharacterLocation(self, charIndex: int, x: float, y: float):
        self.setFaceTileLocation(charIndex, x, y)
        y = math.ceil(y)
        depth = self.__depthMax - y * self.__depthStep
        self.setFaceDepth(charIndex, depth)

    def setCharacterTile(self, charIndex: int, tileX: float, tileY: float):
        self.setFaceTileTexture(charIndex, tileX, tileY)

    def hideCharacter(self, charIndex: int):
        self.hideFace(charIndex)