import numpy as np

from gui.layer import GridLayer
from .OpenGLLayer import OpenGLLayer
from .. import OpenGLGUIFacade


class OpenGLGridLayer(OpenGLLayer, GridLayer):

    def __init__(self, gui: OpenGLGUIFacade):
        super(OpenGLGridLayer, self).__init__(gui)
        self.__width = 0  # type: int
        self.__height = 0  # type: int

    def updateDepths(self, depth: int, index: int, maxIndex: int) -> int:
        faceIndex = 0
        for y in range(self.__height):
            self.setDepths(depth - index - y * maxIndex, range(faceIndex, faceIndex+self.__width))
            faceIndex += self.__width
        return depth - index - self.__height * maxIndex

    def setTiles(self, grid: np.ndarray):
        assert grid.ndim == 3
        self.__width = gridWidth = grid.shape[0]
        self.__height = gridHeight = grid.shape[1]
        assert grid.shape[2] == 2

        self.createFaces(gridWidth * gridHeight)
        faceIndex = 0
        for y in range(gridHeight):
            for x in range(gridWidth):
                self.setFaceTileLocation(faceIndex, x, y)
                self.setFaceTileTexture(faceIndex, grid[x, y, 0], grid[x, y, 1])
                faceIndex += 1

    def setTile(self, x: int, y: int, tileX: int, tileY: int, width: int = 1, height: int = 1):
        faceIndex = x + y * self.__width
        self.setFaceTileLocation(faceIndex, x, y, width, height)
        self.setFaceTileTexture(faceIndex, tileX, tileY, width, height)

