from typing import Union, Tuple, List, Dict

import numpy as np

from gui.layer import UILayer
from gui.impl.opengl import OpenGLGUIFacade
from .OpenGLLayer import OpenGLLayer
from ..private import FrameBox


class OpenGLUILayer(OpenGLLayer, UILayer):

    def __init__(self, gui: OpenGLGUIFacade):
        super(OpenGLUILayer, self).__init__(gui)
        self.__lastId = 0
        self.__frameBoxes = {}  # type: Dict[int,FrameBox]

    def updateDepths(self, depth: int, index: int, maxIndex: int) -> int:
        self.setDepths(depth - index)
        return depth - index - 1

    def createFrameBox(self, x1: float, y1: float, x2: float, y2: float) -> int:
        # Create id for new frame box
        self.__lastId += 1
        frameId = self.__lastId

        # Allocate faces
        faceIndex = self.addFaces(9)
        frame = FrameBox(faceIndex, x1, y1, x2, y2, self.tileWidth, self.tileHeight)
        self.__frameBoxes[frameId] = frame

        # Defines the faces
        frame.show(self)

        return frameId

    def showFrameBox(self, frameId: int):
        self.__frameBoxes[frameId].show(self)

    def hideFrameBox(self, frameId: int):
        self.__frameBoxes[frameId].hide(self)

    def deleteFrameBox(self, frameId: int):

        # Remove faces used by the frame box
        firstIndex = self.__frameBoxes[frameId].faceIndex
        lastIndex = firstIndex + 9
        self.removeFaces(firstIndex, lastIndex)

        # Remove the frame box from the dict
        del self.__frameBoxes[frameId]

        # Update face indexes
        for frameBox in self.__frameBoxes.values():
            if frameBox.faceIndex >= lastIndex:
                frameBox.faceIndex -= 9




