from ..layer import OpenGLLayer


class FrameBox:
    __slots__ = ['faceIndex', 'x1', 'y1', 'x2', 'y2', 'tileWidth', 'tileHeight']

    def __init__(self, faceIndex: int, x1: float, y1: float, x2: float, y2: float,
                 tileWidth: float, tileHeight: float):
        self.faceIndex = faceIndex
        self.x1 = x1
        self.y1 = y1
        self.x2 = x2
        self.y2 = y2
        self.tileWidth = tileWidth
        self.tileHeight = tileHeight

    def show(self, layer: OpenGLLayer):

        # Local variables (for readability)
        faceIndex = self.faceIndex
        x1 = self.x1
        y1 = self.y1
        x2 = self.x2
        y2 = self.y2
        tileWidth = self.tileWidth
        tileHeight = self.tileHeight

        # Top left
        layer.setFacePixelLocation(faceIndex, x1, y1, x1 + tileWidth, y1 + tileHeight)
        layer.setFacePixelTexture(faceIndex, 0, 0, tileWidth, tileHeight)
        faceIndex += 1

        # Top
        layer.setFacePixelLocation(faceIndex, x1 + tileWidth, y1, x2 - tileWidth, y1 + tileHeight)
        layer.setFacePixelTexture(faceIndex, tileWidth, 0, 2 * tileWidth, tileHeight)
        faceIndex += 1

        # Top right
        layer.setFacePixelLocation(faceIndex, x2 - tileWidth, y1, x2, y1 + tileHeight)
        layer.setFacePixelTexture(faceIndex, 2 * tileWidth, 0, 3 * tileWidth, tileHeight)
        faceIndex += 1

        # Right
        layer.setFacePixelLocation(faceIndex, x2 - tileWidth, y1 + tileHeight, x2, y2 - tileHeight)
        layer.setFacePixelTexture(faceIndex, 2 * tileWidth, tileHeight, 3 * tileWidth, 2 * tileHeight)
        faceIndex += 1

        # Bottom right
        layer.setFacePixelLocation(faceIndex, x2 - tileWidth, y2 - tileHeight, x2, y2)
        layer.setFacePixelTexture(faceIndex, 2 * tileWidth, 2 * tileHeight, 3 * tileWidth, 3 * tileHeight)
        faceIndex += 1

        # Bottom
        layer.setFacePixelLocation(faceIndex, x1 + tileWidth, y2 - tileHeight, x2 - tileWidth, y2)
        layer.setFacePixelTexture(faceIndex, tileWidth, 2 * tileHeight, 2 * tileWidth, 3 * tileHeight)
        faceIndex += 1

        # Bottom left
        layer.setFacePixelLocation(faceIndex, x1, y2 - tileHeight, x1 + tileWidth, y2)
        layer.setFacePixelTexture(faceIndex, 0, 2 * tileHeight, tileWidth, 3 * tileHeight)
        faceIndex += 1

        # Left
        layer.setFacePixelLocation(faceIndex, x1, y1 + tileHeight, x1 + tileWidth, y2 - tileHeight)
        layer.setFacePixelTexture(faceIndex, 0, tileHeight, tileWidth, 2 * tileHeight)
        faceIndex += 1

        # Center
        layer.setFacePixelLocation(faceIndex, x1 + tileWidth, y1 + tileHeight, x2 - tileWidth, y2 - tileHeight)
        layer.setFacePixelTexture(faceIndex, tileWidth, tileHeight, 2 * tileWidth, 2 * tileHeight)
        faceIndex += 1

    def hide(self, layer: OpenGLLayer):
        for faceIndex in range(9):
            layer.hideFace(self.faceIndex + faceIndex)

    def __str__(self):
        return "{} ({},{},{},{}) ({},{})".format(
            self.faceIndex,
            self.x1,
            self.y1,
            self.x2,
            self.y2,
            self.tileWidth,
            self.tileHeight
        )