import time
from typing import List

import pygame
from pygame.event import Event

from gui.input import Keyboard, KeyEvent
from gui.impl.pygame import PygameKeyEvent

class PygameKeyboard(Keyboard):

    def __init__(self):
        self.__keys = [False] * Keyboard.K_MAXCODE  # type: List[bool]
        self.__events = []  # type: List[PygameKeyEvent]

    def isKeyPressed(self, key: int) -> bool:
        if key < 0 or key >= len(self.__keys):
            return False
        return self.__keys[key]

    @property
    def keyEvents(self) -> List[KeyEvent]:
        return self.__events

    def clearKeyEvents(self):
        self.__events.clear()

    def parsePygameEvent(self, event: Event):
        if event.type == pygame.KEYDOWN:
            if event.key not in pygameKeyCode2FacadeCode:
                return
            key = pygameKeyCode2FacadeCode[event.key]
            keyEvent = PygameKeyEvent(KeyEvent.KEYDOWN, key, time.time())
            self.__events.append(keyEvent)
            self.__keys[key] = True
        elif event.type == pygame.KEYUP:
            if event.key not in pygameKeyCode2FacadeCode:
                return
            key = pygameKeyCode2FacadeCode[event.key]
            keyEvent = PygameKeyEvent(KeyEvent.KEYUP, key, time.time())
            self.__events.append(keyEvent)
            self.__keys[key] = False


pygameKeyCode2FacadeCode = {
    pygame.K_UNKNOWN: Keyboard.K_UNKNOWN,
    pygame.K_FIRST: Keyboard.K_FIRST,
    pygame.K_BACKSPACE: Keyboard.K_BACKSPACE,
    pygame.K_TAB: Keyboard.K_TAB,
    pygame.K_CLEAR: Keyboard.K_CLEAR,
    pygame.K_RETURN: Keyboard.K_RETURN,
    pygame.K_PAUSE: Keyboard.K_PAUSE,
    pygame.K_ESCAPE: Keyboard.K_ESCAPE,
    pygame.K_SPACE: Keyboard.K_SPACE,
    pygame.K_EXCLAIM: Keyboard.K_EXCLAIM,
    pygame.K_QUOTEDBL: Keyboard.K_QUOTEDBL,
    pygame.K_HASH: Keyboard.K_HASH,
    pygame.K_DOLLAR: Keyboard.K_DOLLAR,
    pygame.K_AMPERSAND: Keyboard.K_AMPERSAND,
    pygame.K_QUOTE: Keyboard.K_QUOTE,
    pygame.K_LEFTPAREN: Keyboard.K_LEFTPAREN,
    pygame.K_RIGHTPAREN: Keyboard.K_RIGHTPAREN,
    pygame.K_ASTERISK: Keyboard.K_ASTERISK,
    pygame.K_PLUS: Keyboard.K_PLUS,
    pygame.K_COMMA: Keyboard.K_COMMA,
    pygame.K_MINUS: Keyboard.K_MINUS,
    pygame.K_PERIOD: Keyboard.K_PERIOD,
    pygame.K_SLASH: Keyboard.K_SLASH,
    pygame.K_0: Keyboard.K_0,
    pygame.K_1: Keyboard.K_1,
    pygame.K_2: Keyboard.K_2,
    pygame.K_3: Keyboard.K_3,
    pygame.K_4: Keyboard.K_4,
    pygame.K_5: Keyboard.K_5,
    pygame.K_6: Keyboard.K_6,
    pygame.K_7: Keyboard.K_7,
    pygame.K_8: Keyboard.K_8,
    pygame.K_9: Keyboard.K_9,
    pygame.K_COLON: Keyboard.K_COLON,
    pygame.K_SEMICOLON: Keyboard.K_SEMICOLON,
    pygame.K_LESS: Keyboard.K_LESS,
    pygame.K_EQUALS: Keyboard.K_EQUALS,
    pygame.K_GREATER: Keyboard.K_GREATER,
    pygame.K_QUESTION: Keyboard.K_QUESTION,
    pygame.K_AT: Keyboard.K_AT,
    pygame.K_LEFTBRACKET: Keyboard.K_LEFTBRACKET,
    pygame.K_BACKSLASH: Keyboard.K_BACKSLASH,
    pygame.K_RIGHTBRACKET: Keyboard.K_RIGHTBRACKET,
    pygame.K_CARET: Keyboard.K_CARET,
    pygame.K_UNDERSCORE: Keyboard.K_UNDERSCORE,
    pygame.K_BACKQUOTE: Keyboard.K_BACKQUOTE,
    pygame.K_a: Keyboard.K_a,
    pygame.K_b: Keyboard.K_b,
    pygame.K_c: Keyboard.K_c,
    pygame.K_d: Keyboard.K_d,
    pygame.K_e: Keyboard.K_e,
    pygame.K_f: Keyboard.K_f,
    pygame.K_g: Keyboard.K_g,
    pygame.K_h: Keyboard.K_h,
    pygame.K_i: Keyboard.K_i,
    pygame.K_j: Keyboard.K_j,
    pygame.K_k: Keyboard.K_k,
    pygame.K_l: Keyboard.K_l,
    pygame.K_m: Keyboard.K_m,
    pygame.K_n: Keyboard.K_n,
    pygame.K_o: Keyboard.K_o,
    pygame.K_p: Keyboard.K_p,
    pygame.K_q: Keyboard.K_q,
    pygame.K_r: Keyboard.K_r,
    pygame.K_s: Keyboard.K_s,
    pygame.K_t: Keyboard.K_t,
    pygame.K_u: Keyboard.K_u,
    pygame.K_v: Keyboard.K_v,
    pygame.K_w: Keyboard.K_w,
    pygame.K_x: Keyboard.K_x,
    pygame.K_y: Keyboard.K_y,
    pygame.K_z: Keyboard.K_z,
    pygame.K_DELETE: Keyboard.K_DELETE,
    pygame.K_KP0: Keyboard.K_KP0,
    pygame.K_KP1: Keyboard.K_KP1,
    pygame.K_KP2: Keyboard.K_KP2,
    pygame.K_KP3: Keyboard.K_KP3,
    pygame.K_KP4: Keyboard.K_KP4,
    pygame.K_KP5: Keyboard.K_KP5,
    pygame.K_KP6: Keyboard.K_KP6,
    pygame.K_KP7: Keyboard.K_KP7,
    pygame.K_KP8: Keyboard.K_KP8,
    pygame.K_KP9: Keyboard.K_KP9,
    pygame.K_KP_PERIOD: Keyboard.K_KP_PERIOD,
    pygame.K_KP_DIVIDE: Keyboard.K_KP_DIVIDE,
    pygame.K_KP_MULTIPLY: Keyboard.K_KP_MULTIPLY,
    pygame.K_KP_MINUS: Keyboard.K_KP_MINUS,
    pygame.K_KP_PLUS: Keyboard.K_KP_PLUS,
    pygame.K_KP_ENTER: Keyboard.K_KP_ENTER,
    pygame.K_KP_EQUALS: Keyboard.K_KP_EQUALS,
    pygame.K_UP: Keyboard.K_UP,
    pygame.K_DOWN: Keyboard.K_DOWN,
    pygame.K_RIGHT: Keyboard.K_RIGHT,
    pygame.K_LEFT: Keyboard.K_LEFT,
    pygame.K_INSERT: Keyboard.K_INSERT,
    pygame.K_HOME: Keyboard.K_HOME,
    pygame.K_END: Keyboard.K_END,
    pygame.K_PAGEUP: Keyboard.K_PAGEUP,
    pygame.K_PAGEDOWN: Keyboard.K_PAGEDOWN,
    pygame.K_F1: Keyboard.K_F1,
    pygame.K_F2: Keyboard.K_F2,
    pygame.K_F3: Keyboard.K_F3,
    pygame.K_F4: Keyboard.K_F4,
    pygame.K_F5: Keyboard.K_F5,
    pygame.K_F6: Keyboard.K_F6,
    pygame.K_F7: Keyboard.K_F7,
    pygame.K_F8: Keyboard.K_F8,
    pygame.K_F9: Keyboard.K_F9,
    pygame.K_F10: Keyboard.K_F10,
    pygame.K_F11: Keyboard.K_F11,
    pygame.K_F12: Keyboard.K_F12,
    pygame.K_F13: Keyboard.K_F13,
    pygame.K_F14: Keyboard.K_F14,
    pygame.K_F15: Keyboard.K_F15,
    pygame.K_NUMLOCK: Keyboard.K_NUMLOCK,
    pygame.K_CAPSLOCK: Keyboard.K_CAPSLOCK,
    pygame.K_SCROLLOCK: Keyboard.K_SCROLLOCK,
    pygame.K_RSHIFT: Keyboard.K_RSHIFT,
    pygame.K_LSHIFT: Keyboard.K_LSHIFT,
    pygame.K_RCTRL: Keyboard.K_RCTRL,
    pygame.K_LCTRL: Keyboard.K_LCTRL,
    pygame.K_RALT: Keyboard.K_RALT,
    pygame.K_LALT: Keyboard.K_LALT,
    pygame.K_RMETA: Keyboard.K_RMETA,
    pygame.K_LMETA: Keyboard.K_LMETA,
    pygame.K_LSUPER: Keyboard.K_LSUPER,
    pygame.K_RSUPER: Keyboard.K_RSUPER,
    pygame.K_MODE: Keyboard.K_MODE,
    pygame.K_HELP: Keyboard.K_HELP,
    pygame.K_PRINT: Keyboard.K_PRINT,
    pygame.K_SYSREQ: Keyboard.K_SYSREQ,
    pygame.K_BREAK: Keyboard.K_BREAK,
    pygame.K_MENU: Keyboard.K_MENU,
    pygame.K_POWER: Keyboard.K_POWER,
    pygame.K_EURO: Keyboard.K_EURO,
    pygame.K_LAST: Keyboard.K_LAST
}
