import pygame

from . import CharacterRenderer
from gui.text import TextStyle


class ShadowCharacterRenderer(CharacterRenderer):

    def __init__(self, style: TextStyle, characterRenderer: CharacterRenderer, shadowCharacterRenderer: CharacterRenderer):
        self.__shadowShift = style.shadowShift
        self.__characterRenderer = characterRenderer
        self.__shadowCharacterRenderer = shadowCharacterRenderer

    @property
    def tileDefaultWidth(self) -> int:
        return self.__characterRenderer.tileDefaultWidth + self.__shadowShift

    @property
    def tileDefaultHeight(self) -> int:
        return self.__characterRenderer.tileDefaultHeight + self.__shadowShift

    def getCharacterAdvance(self, character: str) -> int:
        return self.__characterRenderer.getCharacterAdvance(character)

    def render(self, text: str) -> pygame.Surface:
        surface = self.__characterRenderer.render(text)
        surface2 = self.__shadowCharacterRenderer.render(text)

        # Create a larger surface
        shift = self.__shadowShift
        size = (surface.get_width() + shift, surface.get_height() + shift)
        shadowedSurface = pygame.Surface(size, flags=pygame.SRCALPHA)

        # Blit the shadow and the character
        shadowedSurface.blit(surface2, (shift, shift))
        shadowedSurface.blit(surface, (0, 0))

        return shadowedSurface
