from __future__ import annotations

from state import State, Direction
from logic.command.Command import Command, CHARACTER_DIRECTION_PRIORITY
from logic import Logic


class CharacterLocationCommand(Command):

    def __init__(self, state: State, regionName: str, characterIndex: int, x: int, y: int):
        self.__state = state
        self.__regionName = regionName
        self.__characterIndex = characterIndex
        self.__x = x
        self.__y = y

    def priority(self) -> int:
        return CHARACTER_DIRECTION_PRIORITY + self.__characterIndex

    def execute(self, logic: Logic):
        if self.__regionName is None:
            return
        region = self.__state.findRegion(self.__regionName)
        if region is None:
            return
        character = region.findCharacter(self.__characterIndex)
        if character is None:
            return

        character.displayDirection = Direction.SOUTH
        character.moveDirection = Direction.NONE
        character.commandDirection = Direction.NONE
        character.x = self.__x
        character.y = self.__y
        character.shift = 0

        self.__state.notifyCharacterChanged(self.__state, self.__regionName, self.__characterIndex)



