from __future__ import annotations

import random

from logic import Logic
from state import State
from state.Region import TREE_FLOWER_1, TREE_HERB_1, TREE_MUSHROOM_1, TREE_NENUPHAR_1, REGION_MAX_WIDTH
from logic.command.Command import Command, REGION_PRIORITY


class FillTreeValueCommand(Command):

    def __init__(self, state: State, regionName: str, x: int, y: int, background: int, foreground: int):
        self.__state = state
        self.__regionName = regionName
        self.__x = x
        self.__y = y
        self.__background = background
        self.__foreground = foreground

    def priority(self) -> int:
        return REGION_PRIORITY + self.__x + self.__y * REGION_MAX_WIDTH

    def execute(self, logic: Logic):
        region = self.__state.getRegion(self.__regionName)
        if not (0 <= self.__x < region.width and 0 <= self.__y < region.height):
            return
        currentValue = region.getTreeValue(self.__x, self.__y)

        value = self.__foreground
        if TREE_FLOWER_1 <= value <= TREE_FLOWER_1 + 3:
            if TREE_FLOWER_1 <= currentValue <= TREE_FLOWER_1 + 3:
                return
            value = TREE_FLOWER_1 + random.randint(0, 3)
        elif TREE_HERB_1 <= value <= TREE_HERB_1 + 3:
            if TREE_HERB_1 <= currentValue <= TREE_HERB_1 + 3:
                return
            value = TREE_HERB_1 + random.randint(0, 3)
        elif TREE_MUSHROOM_1 <= value <= TREE_MUSHROOM_1 + 1:
            if TREE_MUSHROOM_1 <= currentValue <= TREE_MUSHROOM_1 + 1:
                return
            value = TREE_MUSHROOM_1 + random.randint(0, 1)
        elif TREE_NENUPHAR_1 <= value <= TREE_NENUPHAR_1 + 1:
            if TREE_NENUPHAR_1 <= currentValue <= TREE_NENUPHAR_1 + 1:
                return
            value = TREE_NENUPHAR_1 + random.randint(0, 1)
        if currentValue == value or currentValue != self.__background:
            return
        region.setTreeValue(self.__x, self.__y, value)
        self.__state.notifyRegionCellChanged(self.__regionName, self.__x, self.__y)

        logic.addCommand(FillTreeValueCommand(self.__state, self.__regionName, self.__x + 1, self.__y, self.__background, self.__foreground))
        logic.addCommand(FillTreeValueCommand(self.__state, self.__regionName, self.__x - 1, self.__y, self.__background, self.__foreground))
        logic.addCommand(FillTreeValueCommand(self.__state, self.__regionName, self.__x, self.__y + 1, self.__background, self.__foreground))
        logic.addCommand(FillTreeValueCommand(self.__state, self.__regionName, self.__x, self.__y - 1, self.__background, self.__foreground))
