from gui import GUIFacade
from gui.input import Keyboard
from gui.text import TextStyle
from logic.command import CreateRegionCommand, CharacterDirectionCommand, MoveCharacterCommand, LoadStateCommand
from state import State, Direction
from .DefaultGameMode import DefaultGameMode


class PlayGameMode(DefaultGameMode):

    def __init__(self, gui: GUIFacade):
        super(PlayGameMode, self).__init__(gui)

        self.__frontLayerGroup = gui.createLayerGroup()
        self.__framesLayer = self.__frontLayerGroup.createUILayer()
        self.__framesLayer.setTileset("assets/ui/ui.png")
        self.__framesLayer.setTileSize(self._tileSize, self._tileSize)
        self.__framesLayer.createFrameBox(
            self._tileSize,
            gui.screenHeight - self._tileSize * 2,
            gui.screenWidth - self._tileSize,
            gui.screenHeight,
        )

        style = TextStyle("assets/font/basis33/basis33.ttf", self._tileSize // 2, (255, 255, 255))
        self.__textLayer = self.__frontLayerGroup.createTextLayer()
        self.__textLayer.setStyle(style)
        self.__textLayer.setMaxCharacterCount(500)
        self.__textLayer.setText(
            self._tileSize * 1.5,
            gui.screenHeight - self._tileSize * 1.75,
            "Move around with the arrows: you can hide behind the trees!\n"
            "This is thanks to the OpenGL Z-Buffer.\n"
            "More details https://www.patternsgameprog.com"
        )

    #############################################################
    # Mode
    #############################################################

    def init(self):
        self._logic.addCommand(LoadStateCommand(self, 'level.bin'))

    def handleInputs(self) -> bool:

        # Keyboard
        keyboard = self._gui.keyboard
        direction = Direction.NONE
        if keyboard.isKeyPressed(Keyboard.K_LEFT):
            direction = Direction.WEST
        elif keyboard.isKeyPressed(Keyboard.K_RIGHT):
            direction = Direction.EAST
        elif keyboard.isKeyPressed(Keyboard.K_UP):
            direction = Direction.NORTH
        elif keyboard.isKeyPressed(Keyboard.K_DOWN):
            direction = Direction.SOUTH
        if direction != Direction.NONE:
            self._logic.addCommand(CharacterDirectionCommand(self._state, self._currentRegionName, 0, direction))
            return True

        return False

    def render(self, time: float, epochFrame: float):
        self._worldRenderer.updateAnimations(time, epochFrame)
        self._worldRenderer.updateViewOnCharacter(0)
        self._gui.render()

