from gui.layer import CharactersLayer
from state import Character, Direction


class CharacterAnimation:

    def __init__(self, layer: CharactersLayer, index: int, character: Character):
        self.__layer = layer
        self.__index = index
        self.__character = character
        self.__x = 0
        self.__y = 0

    def getLocation(self) -> (float, float):
        return self.__x, self.__y

    def update(self, time: float, epochFrame: float):
        x = self.__character.x
        y = self.__character.y
        shift = self.__character.shift
        if shift > 0:
            shift = (shift - 1 + epochFrame) / self.__character.speed - 1.0
        direction = self.__character.moveDirection
        tileStep = round(3 * shift) % 3

        if direction == Direction.NORTH:
            y -= shift
        elif direction == Direction.SOUTH:
            y += shift
        elif direction == Direction.WEST:
            x -= shift
        elif direction == Direction.EAST:
            x += shift
        else:
            direction = self.__character.displayDirection
            tileStep = 1

        self.__x = x
        self.__y = y
        self.__layer.setCharacterLocation(self.__index, x, y)

        if direction == Direction.NORTH:
            tileX = 9
        elif direction == Direction.WEST:
            tileX = 3
        elif direction == Direction.EAST:
            tileX = 6
        else:
            tileX = 0
        tileY = self.__character.category - 1

        self.__layer.setCharacterTile(self.__index, tileX + tileStep, tileY)

