import argparse
import logging

from gui import GUIFacadeFactory
from gui.input import KeyEvent, Keyboard
from mode import PlayGameMode, EditGameMode

logging.basicConfig(level=logging.INFO, format='\r%(filename)s:%(lineno)d: %(message)s')

# Parse arguments
parser = argparse.ArgumentParser()
parser.add_argument("--editor", action='store_true')
parser.add_argument("--fullScreen", action='store_true')
parser.set_defaults(editor=False, fullScreen=False)
params = parser.parse_args()

# Create window
gui = GUIFacadeFactory().createInstance("OpenGL")

# Create a play game mode
if params.editor:
    gui.createWindow(
        "OpenGL 2D Facade - https://www.patternsgameprog.com/",
        1280, 768, fullScreen=params.fullScreen
    )
    gui.init()
    mainMode = EditGameMode(gui)
else:
    gui.createWindow(
        "OpenGL 2D Facade - https://www.patternsgameprog.com/",
        640, 360, fullScreen=params.fullScreen
    )
    gui.init()
    mainMode = PlayGameMode(gui)

# Run game
mainMode.init()
frameCount = 0
epochCount = 0
epochRate = 5
while not gui.closingRequested:

    # Handle inputs
    gui.updateInputs()
    if not mainMode.handleInputs():
        # If the mode didn't handle inputs, run a default handling
        keyboard = gui.keyboard
        for keyEvent in keyboard.keyEvents:
            if keyEvent.type == KeyEvent.KEYDOWN:
                if keyEvent.key == Keyboard.K_ESCAPE:
                    gui.closingRequested = True
                    break
        keyboard.clearKeyEvents()

    # Update game state
    if (frameCount % epochRate) == 0:
        mainMode.updateState()
        epochCount += 1

    # Render scene
    mainMode.render(gui.getTime(), (frameCount % epochRate) / epochRate)

    frameCount += 1

# Delete objects
gui.quit()



